
import sys

if 1:
   # binutils: 2.37
   # GCC: 11.2.0
   # PyQt: 6.2.2
   # Python: 3.9.7
   # Qt: 6.2.2
   # sip: 6.5.0
   import PyQt6 as PQt
   import PyQt6.QtCore
   import PyQt6.QtWidgets
else:
   # binutils: 2.35
   # GCC: 10.2.0
   # PyQt: 5.15.1
   # Python: 3.8.5
   # Qt: 5.15.1
   # sip: 5.4.0
   import PyQt5 as PQt
   import PyQt5.QtCore
   import PyQt5.QtWidgets

#-------------------------------------------------------------
def DBG( msg ):
   print( f'DEBUG: {msg}' )
   sys.stdout.flush()

#-------------------------------------------------------------
def handleItemChanged( item, col ):
   DBG( f"An item in column {col} was changed" )

#-------------------------------------------------------------
class CheckEditor( PQt.QtWidgets.QWidget ):

   valueUpdated = PQt.QtCore.pyqtSignal( PQt.QtWidgets.QWidget )

   #-----------------------------------------------------------------------
   def __init__( self, parent = None ):
      """Initialize the button."""
      super().__init__( parent )
      self.widget = PQt.QtWidgets.QCheckBox( self )

      layout = PQt.QtWidgets.QGridLayout( self )
      layout.setSpacing( 0 )
      layout.setContentsMargins( 0, 2, 0, 0 )
      layout.addWidget( self.widget )

      self.widget.stateChanged.connect( self.widgetCb )

   #-----------------------------------------------------------------------
   def widgetCb( self ):
      """Handle the button click."""
      DBG( "the CheckEditor button was pushed" )
      self.valueUpdated.emit( self )

#-------------------------------------------------------------
class BoolDelegate( PQt.QtWidgets.QStyledItemDelegate ):

   CheckRole = PQt.QtCore.Qt.ItemDataRole.UserRole + 1
   LabelRole = PQt.QtCore.Qt.ItemDataRole.UserRole + 2

   #-----------------------------------------------------------------------
   def __init__( self, parent ):
      # Initialize the Item
      super().__init__( parent )

   #-----------------------------------------------------------------------
   def createEditor( self, parent, styleOption, modelIndex ):
      """Create an editor widget that will appear in the table cell.
      """
      DBG( 'creating the editor' )
      editor = CheckEditor( parent )
      editor.valueUpdated.connect( self.emitCommitData )
      return editor

   #-----------------------------------------------------------------------
   def setEditorData( self, editor, modelIndex ):
      """initialize the editor widget.
      """
      DBG( 'initializing the editor data' )
      txt = modelIndex.model().data( modelIndex, self.LabelRole )
      editor.widget.setText( txt )

      state = modelIndex.model().data( modelIndex, self.CheckRole )
      editor.widget.setCheckState( state )

   #-----------------------------------------------------------------------
   def setModelData( self, editor, model, modelIndex ):
      """Set the data with that from the editor widget.
      """
      state = editor.widget.checkState()
      DBG( f'setData state = {state}' )

      result = model.setData( modelIndex, state, self.CheckRole )
      DBG( f'setData result = {result}' )
      #ERROR: The result will always be False because this calls
      #ERROR: 'QAbstractItemModel::setData' instead of 'QTreeModel::setData'
      #ERROR: This results in the model not getting updated or
      #ERROR: emitting the proper signal

      # calling 'setItemData' works because the base-class can properly
      # access the virtual table and call the QTreeModel::setData method.
      #WORKS: model.setItemData( modelIndex, { self.CheckRole : state } )

   #-----------------------------------------------------------------------
   def emitCommitData( self, editor ):
      """Signal that the editor is finished and that the data needs to
         be committed.
      """
      DBG( "editing is completed" )
      self.commitData.emit( editor )

#=============================================================
qApp = PQt.QtWidgets.QApplication( [] )

#NOTE: This problem exists for both QTreeWidget and QListWidget
tree = PQt.QtWidgets.QTreeWidget( None )

delegate = BoolDelegate( tree )
tree.setItemDelegateForColumn( 0, delegate )

item = PQt.QtWidgets.QTreeWidgetItem( tree )
item.setFlags( PQt.QtCore.Qt.ItemFlag.ItemIsSelectable |
               PQt.QtCore.Qt.ItemFlag.ItemIsEditable |
               PQt.QtCore.Qt.ItemFlag.ItemIsEnabled )

item.setData( 0, BoolDelegate.LabelRole, "Test Item" )
item.setData( 0, BoolDelegate.CheckRole, PQt.QtCore.Qt.CheckState.Checked )

tree.openPersistentEditor( item, 0 )

tree.itemChanged.connect( handleItemChanged )
tree.show()

# run the script interactively
#    $> python -i test_delegate.py
# click check box (should toggle off)
# -- note that 'setModelData' fails and editing does not complete
# confirm the state was not updated
#    >>> item.data( 0, BoolDelegate.CheckRole )
# -- Will report as 'Checked'

